<?php

namespace App\Command;

use App\Entity\ModelMetadata\ClusterCommand;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class ClusterPublishWaitCommand extends Command
{
    protected $container;

    public function __construct(ContainerInterface $container) {
        $this->container = $container;
        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('app:cluster:publish:wait')
            ->setAliases(['cybersponse:cluster:publish:wait'])
            ->setDescription('Waiting for Model File Generation on the other nodes.');
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);
        $errorIo = $io;
        $sequenceId = substr(str_shuffle("0123456789abcdefghijklmnopqrstuvwxyz"), 0, 10);
        $dasUri = $this->container->getParameter('das.uri');
        $dasClient = $this->container->get("app.proxy.client.das");
        $objectData = array(
            'commandId' => 'cluster_publish',
            'command' => 'sudo -u nginx php /opt/cyops-api/bin/console app:cluster:publish --no-interaction --nodeId=$nodeId$ --sequenceId=' . $sequenceId,
            'includeFiredrillNodes' => false
        );
        $headers = array(
            "Content-type" => "application/json"
        );
        $dasRequests = new GuzzleRequest('post', $dasUri . '/clustercommand/', $headers, json_encode($objectData));
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $resCode = $dasResponse->getStatusCode();
            if ($resCode != 200){
                return -1;
            }
        } catch (\Exception $error) {
            $errorIo->error($error->getMessage());
            return -1;
        }

        $dasRequests = new GuzzleRequest('get', $dasUri . '/cluster/?self=false', $headers);

        $nodeIds = [];
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $dasResponseData = $dasResponse->getBody()->getContents();
            $jsonDecodeResponse = json_decode($dasResponseData);
            $nodes = $jsonDecodeResponse->{"nodes"};
            foreach ($nodes as $node){
                if (strpos(strtolower($node->{"comment"}), "firedrill in progress") === FALSE){
                    array_push($nodeIds, $node->{"nodeId"});
                }
            }

        } catch (\Exception $error) {
            $errorIo->error($error->getMessage());
            return -1;
        }

        $container = $this->container;
        $entityManager = $container->get('doctrine.orm.entity_manager');
        foreach ($nodeIds as &$nodeId) {
            $clusterCommand = new ClusterCommand();
            $clusterCommand->setNodeId($nodeId);
            $clusterCommand->setCommandId("cybersponse:cluster:publish");
            $clusterCommand->setSequenceId($sequenceId);
            $clusterCommand->setStatus('todo');
            $entityManager->persist($clusterCommand);
        }
        $entityManager->flush();

        $clusterCommandRepository = $entityManager->getRepository(ClusterCommand::class);
        # wait for 10 mins for other cluster nodes to complete
        $counter = 60;
        $waitForStatus = count($clusterCommandRepository->findAll([
            'sequenceId' => $sequenceId,
            'status' => 'todo']));
        while ($waitForStatus > 1 && $counter > 0) {
            sleep(10);
            $counter = $counter - 1;
            $waitForStatus = count($clusterCommandRepository->findAll([
                'sequenceId' => $sequenceId,
                'status' => 'todo']));
            }
        $io->success('Finished with publish across cluster!');
        return 0;
    }
}
