<?php

namespace App\Command;

use App\Entity\Core\PgFile;
use App\Entity\Core\Widget;
use App\Service\UtilityService;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class BuildWidgetFilesCommand
 */
class WidgetsBuildCommand extends Command
{
    const WIDGET_PATH = '/opt/cyops-ui/widgets';

    protected $container;
    private $utilityService;

    public function __construct(ContainerInterface $container, UtilityService $utilityService)
    {
        $this->container = $container;
        $this->utilityService = $utilityService;
        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('app:widgets:build')
            ->setAliases(['cybersponse:widgets:build'])
            ->setDescription('Load all widgets from the widget module, build the widget cache file, and copy to exo-ui.')
            ->setDefinition([
              new InputOption(
                  'widgetUuId',
                  'w',
                  InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY,
                  'Arguments for building the widget with uuId.'
              ),
              new InputOption(
                  'delete',
                  'd',
                  InputOption::VALUE_OPTIONAL | InputOption::VALUE_IS_ARRAY,
                  'Arguments for the widget delete.'
              ),
              new InputOption(
                  'rebuildAll',
                  'r',
                  InputOption::VALUE_OPTIONAL | InputOption::VALUE_IS_ARRAY,
                  'Arguments for rebuilding all widgets.'
              )
          ]);
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $output->writeln([
        '===========================================================',
        'Starting Widgets build',
        '===========================================================',
        '',
    ]);
        $container = $this->container;
        $widgetsUuid = $input->getOption('widgetUuId');
        $deleteWidget= $input->getOption('delete');

        $widgetManager = $container->get('doctrine.orm.entity_manager');

        /** @var EntityRepository $widgetRepository */
        $widgetRepository = $widgetManager->getRepository(Widget::class);

        /** @var widgets[] $widgets */
        $widgets = $widgetRepository->findAll();
        foreach ($widgets as $widget) {
            if (in_array($widget->getUuid(), $widgetsUuid)|| in_array($widget->getId(), $widgetsUuid)) {
                $output->writeln([
                '===========================================================',
                'Generating files for '.$widget->getName().'-'.$widget->getVersion().' Widget',
                '===========================================================',
                '',
              ]);
                $this->utilityService->buildWidgetFiles($widget, self::WIDGET_PATH, $container->get('doctrine')->getManagerForClass(PgFile::class));
            } elseif (in_array($widget->getUuid(), $deleteWidget)|| in_array($widget->getId(), $deleteWidget)) {
                $this->utilityService->removeWidgetFiles($widget);
            } elseif (count($input->getOption('rebuildAll'))) {
                $output->writeln([
                  '===========================================================',
                  'Rebuilding '.$widget->getName().'-'.$widget->getVersion().' Widget',
                  '===========================================================',
                  '',
                ]);
                $this->utilityService->buildWidgetFiles($widget, self::WIDGET_PATH,$container->get('doctrine')->getManagerForClass(PgFile::class));
            }
        }
        $output->writeln([
          '===========================================================',
          'Widgets build finished',
          '===========================================================',
          '',
      ]);
      return 0;
    }
}
