<?php

namespace App\Controller;

use App\Utility\Resolver;
use App\Entity\Custom\Person;
use App\Service\MetadataHelper;
use Symfony\Component\Yaml\Yaml;
use App\Service\SamlActorService;
use App\Controller\BaseController;
use App\Entity\Authorization\Role;
use App\Entity\Authorization\Team;
use App\Entity\Core\EmailTemplate;
use App\Entity\Settings\SystemSettings;
use Doctrine\ORM\EntityManagerInterface;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\HttpFoundation\Request;
use GuzzleHttp\Exception\BadResponseException;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Contracts\Cache\TagAwareCacheInterface;
use ApiPlatform\Core\Api\ResourceClassResolverInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

class PublicActionController extends BaseController
{
    protected $samlActorService;

    protected $entityManager;

    public function __construct(ContainerInterface $containers, SamlActorService $samlActorService, ResourceClassResolverInterface $resourceClassResolver,
        TagAwareCacheInterface $cache, MetadataHelper $metadataHelper, EntityManagerInterface $entityManager)
    {
        $this->containers = $containers;
        $this->samlActorService = $samlActorService;
        $this->entityManager = $entityManager;
        parent::__construct($resourceClassResolver, $containers, $cache, $metadataHelper);
    }

    /**
     * @return integrationsClient
     */
    public function getIntegrationClient()
    {
        return $this->containers->get("app.proxy.client.integrations");
    }

    /**
     * @return DasClient
     */
    public function getDasClient()
    {
        return $this->containers->get("app.proxy.client.das");
    }

    /**
     * @return ProxyClient
     */
    public function getProxyClient()
    {
        return $this->containers->get("app.proxy.client.workflow");
    }

    public function sendMail($templateId, $dataToReplace, $mailDetails, $integrationClient, $defaultMessage)
    {
        $entityRepository = $this->entityManager->getRepository(EmailTemplate::class);
        $criteria = array('uuid' => $templateId);
        $item = $entityRepository->find($criteria);
        if ($item == null) {
            $email_options = Yaml::parse(file_get_contents('/opt/cyops/configs/cyops-api/mailtemplate.yml'));
            $item = $email_options[$defaultMessage];
        } else {
            $context['ignore_authorization'] = true;
            $context['ignore_field_authorization'] = true;
            $item = $this->get('serializer')->normalize($item, 'jsonld', $context);
        }

        $headers = array(
            "Content-type" => "application/json",
        );
        if (strpos($item['content'], 'https://{{@Server_fqhn}}')) {
            $item['content'] = str_replace('https://{{@Server_fqhn}}', '{{@Server_fqhn}}', $item['content']);
        }
        if (strpos($item['content'], 'https://{{@cyops_hostname}}')) {
            $item['content'] = str_replace('https://{{@cyops_hostname}}', '{{@cyops_hostname}}', $item['content']);
        }
        foreach ($dataToReplace as $key => $value) {
            $item['content'] = str_replace($key, $value, $item['content']);
        }
        $smtpObject = '{
                            "connector": "smtp",
                            "operation": "send_richtext_email",
                            "params": {
                                "ccc": "",
                                "bcc": "",
                                "content_type":"text/html"
                            }
                        }';
        $jsonDecodedSmtp = json_decode($smtpObject, true);
        $jsonDecodedSmtp['params']['subject'] = $item['subject'];
        $jsonDecodedSmtp['params']['content'] = $item['content'];
        $jsonDecodedSmtp['params']['to'] = $mailDetails;
        $smtpUri = $this->containers->getParameter('integrations.uri');
        $smtpUri = $smtpUri . '/execute/?format=json';
        $smtpRequest = new GuzzleRequest('post', $smtpUri, $headers, json_encode($jsonDecodedSmtp));
        try {
            $integrationClient->send($smtpRequest);
        } catch (\Exception $exception) {
            error_log($exception->getMessage());
            $errVar = json_decode($exception->getResponse()->getBody()->read(2048), true);
            error_log($errVar);
            return $errVar;
        }
        return true;
    }

    public function routeForgotPasswordAction(Request $request)
    {
        $objectData = json_decode($request->getContent(), true);
        $token = $objectData['captchResponse'];
        $success = $this->verifyReCaptcha($token);
        if (!$success) {
            return Response::create('Invalid Captcha, please verify', Response::HTTP_FORBIDDEN);
        }
        $loginId = $objectData['loginId'];
        $userId = $this->getUserId($loginId);

        if (empty($userId)) {
            return new JsonResponse(["message" => "User with loginid '" . $loginId . "' not found"], Response::HTTP_NOT_FOUND);
        }

        $email = $this->getEmail($userId);
        $data = array(
            'uuid' => $userId,
            'newUser' => false,
            'email' => $email,
        );

        $dasClient = $this->getDasClient();
        $headers = array(
            "Content-type" => "application/json",
        );

        $dasUri = $this->containers->getParameter('das.uri');
        $keyFile = file_get_contents('/opt/cyops-api/src/keys/daspublic.key', true);
        $idKey = base64_encode($keyFile);
        $data['idKey'] = $idKey;
        $dasRequests = new GuzzleRequest('post', $dasUri . '/token?uuid=' . $data['uuid'], $headers, json_encode($data));
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $dasResponseData = $dasResponse->getBody()->getContents();
            $jsonDecodeResponse = json_decode($dasResponseData, true);
            $resCode = $dasResponse->getStatusCode();
        } catch (\Exception $error) {
            error_log($error->getMessage());
            $errVar = json_decode($error->getResponse()->getBody()->read(2048), true);
            $errMsg['message'] = $errVar['Error'];
            $errCode = $error->getCode();
            return JsonResponse::create($errMsg, $errCode);
        }

        if ($resCode < 300) {
            $managerRegistry = $this->containers->get('doctrine');
            $manager = $managerRegistry->getManagerForClass(SystemSettings::class);
            $systemRepository = $manager->getRepository(SystemSettings::class);
            $systemSetting = $systemRepository->findAll();
            $global_value = $systemSetting[0]->getGlobalValues();
            $system_host = isset($global_value['hostname']) ? $global_value['hostname'] : $request->getHttpHost();

            $responseCode = 400;
            $integrationClient = $this->getIntegrationClient();
            $dataToReplace = [];
            $dataToReplace['{{vars.token}}'] = $jsonDecodeResponse['token'];
            $dataToReplace['{{vars.request.baseUri}}'] = $request->getScheme() . "://" . $system_host;
            $dataToReplace['{{@Server_fqhn}}'] = $request->getScheme() . "://" . $system_host;
            $dataToReplace['{{@cyops_hostname}}'] = $request->getScheme() . "://" . $system_host;
            $defaultMessage = 'forgotpassword';
            $mailResponse = $this->sendMail('b9265a6e-7c0d-420a-a8e9-e77f01baf7cb', $dataToReplace, $data['email'], $integrationClient, $defaultMessage);
            if (!(is_array($mailResponse)) && $mailResponse === true) {
                $mailResponse = "Reset token sent to user";
                $responseCode = 200;
            }
            return JsonResponse::create($mailResponse, $responseCode);
        }
    }

    public function routeLicenseAction(Request $request)
    {
        $objectData = json_decode($request->getContent(), true);
        $objectData['type'] = 'unauthenticated';
        $objectData['action'] = $objectData['action'] ?? "deploy_license";
        $dasClient = $this->getDasClient();
        $headers = array(
            "Content-type" => "application/json",
            "X-REMOTE_ADDR" => $_SERVER['REMOTE_ADDR'],
        );

        $dasUri = $this->containers->getParameter('das.uri');
        $dasRequests = new GuzzleRequest('post', $dasUri . '/license', $headers, json_encode($objectData));
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $dasResponseData = $dasResponse->getBody()->getContents();
            $jsonDecodeResponse = json_decode($dasResponseData, true);
            $resCode = $dasResponse->getStatusCode();
            return JsonResponse::create($jsonDecodeResponse, $resCode);
        } catch (\Exception $error) {
            error_log($error->getMessage());
            $errVar = json_decode($error->getResponse()->getBody()->read(2048), true);
            $errMsg['message'] = $errVar['Error'];
            $errCode = $error->getCode();
            return JsonResponse::create($errMsg, $errCode);
        }
    }

    /**
     * @param $userId
     * @return string
     */
    private function getEmail($userId)
    {
        $actor = $this->samlActorService->getActor($userId);
        $email = $actor->getEmail();
        return $email;
    }

    /**
     * @param $token
     * @return bool
     */
    private function verifyReCaptcha($token)
    {
        $googleAPI = $this->containers->getParameter('googleAPI');
        $googleSecrete = $this->containers->getParameter('secretKey');

        $post_data = array(
            'form_params' => [
                'secret' => $googleSecrete,
                'response' => $token]);

        $client = new \GuzzleHttp\Client();
        $responseObject = $client->request('POST', $googleAPI, $post_data);
        if ($responseObject->getStatusCode() >= 200 && $responseObject->getStatusCode() < 300) {
            $result = json_decode($responseObject->getBody());
            return $result->success;
        } else {
            return false;
        }
    }

    /**
     * @param $routeIdentifier
     * @return array
     */
    private function findHandlerMapForRoute($routeIdentifier)
    {
        $handlerMap = $this->containers->getParameter('app.proxy.handler.map');

        if (!array_key_exists($routeIdentifier, $handlerMap)) {
            throw new NotFoundHttpException();
        }
        return $handlerMap;
    }

    /**
     * @param $routeIdentifier
     * @return ProxyClient
     */
    public function findHandlerForRoute($routeIdentifier)
    {
        $handler = $this->findHandlerMapForRoute($routeIdentifier);

        return $this->containers->get($handler[$routeIdentifier]['handlerName']);
    }

    /**
     * @param String $loginId
     * @return String
     */
    private function getUserId($loginId)
    {
        $client = $this->findHandlerForRoute('auth');
        $body = ["json" => ["loginid" => $loginId]];
        $userRequestUri = sprintf("%s/%s", $client->getTargetUri(), 'users');
        $response = null;
        try {
            $response = $client->request('POST', $userRequestUri, $body);
            $respBody = json_decode($response->getBody()->getContents(), true);
            return $respBody['uuid'];
        } catch (BadResponseException $exception) {
            return "";
        }
    }

    public function portalUserAction(Request $request)
    {

        $managerRegistry = $this->containers->get('doctrine');
        $manager = $managerRegistry->getManagerForClass(SystemSettings::class);
        $systemRepository = $manager->getRepository(SystemSettings::class);
        $systemSetting = $systemRepository->findAll();
        $global_value = $systemSetting[0]->getGlobalValues();
        $host = isset($global_value['hostname']) ? $global_value['hostname'] : $_SERVER['HTTP_HOST'];
        $dasClient = $this->getDasClient();
        $request_headers = $request->headers->all();
        $cookie = $request_headers["Cookie"] ?? $request_headers["cookie"] ?? null;
        $request_headers["Cookie"] = $cookie;
        $request_headers["EXTENSION_CONTAINER_HOSTNAME"] = $host;

        $dasUri = $this->containers->getParameter('das.uri');
        $dasRequests = new GuzzleRequest('put', $dasUri . '/token', $request_headers, $request->getContent());
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $dasResponseData = $dasResponse->getBody()->getContents();
            $jsonDecodeResponse = json_decode($dasResponseData, true);
            $resCode = $dasResponse->getStatusCode();
        } catch (\Exception $error) {
            error_log($error->getMessage());
            $errVar = json_decode($error->getResponse()->getBody()->read(2048), true);
            $errMsg['message'] = $errVar['Error'] ?? null;
            $errCode = $error->getCode();
            return JsonResponse::create($errMsg, $errCode);
        }

        if ($resCode < 300) {
            if ($jsonDecodeResponse['user_details']) {
                $personObject = new Person();
                $personObject->setFirstname(explode(' ', $jsonDecodeResponse['user_details']['user_full_name'])[0]);
                $personObject->setLastname(explode(' ', $jsonDecodeResponse['user_details']['user_full_name'])[1]);
                $personObject->setUserId($jsonDecodeResponse['user_details']['user_id']);
                $personObject->setCsActive(true);
                $personObject->setEmail($jsonDecodeResponse['user_details']['user_email']);
                $managerRegistry = $this->containers->get('doctrine');
                $roles = $managerRegistry->getManagerForClass(Role::class)->getRepository(Role::class)->findAll();
                foreach ($roles as $key => $value) {
                    if (\in_array($value->getUuid(), $jsonDecodeResponse['user_details']['roles'])) {
                        $personObject->addRole($value);
                    }
                }
                $teams = $managerRegistry->getManagerForClass(Team::class)->getRepository(Team::class)->findAll();
                foreach ($teams as $key => $value) {
                    if (\in_array($value->getUuid(), $jsonDecodeResponse['user_details']['teams'])) {
                        $personObject->addTeam($value);
                    }
                }
                $manager = $managerRegistry->getManagerForClass(Person::class);
                $manager->persist($personObject);
                $manager->flush();
            }
            $post_params = $jsonDecodeResponse['post_params'] ?? null;
            $portals =  $jsonDecodeResponse['portals'] ?? null;
            $logout_url =  $jsonDecodeResponse['logout_url'] ?? null;
            return JsonResponse::create(array('token' => $jsonDecodeResponse['new_token'], 'post_params' => $post_params,
                                        'portals' => $portals, 'logout_url' => $logout_url));
        }
    }

    public function getAction()
    {
        $path = 'authentication';
        $documentManagerRegistry = $this->getDoctrine();
        $systemSettingsManager = $documentManagerRegistry->getManagerForClass(SystemSettings::class);

        /** @var SystemSettingsRepository $systemSettingsRepository */
        $systemSettingsRepository = $systemSettingsManager->getRepository(SystemSettings::class);
        $systemSettings = $systemSettingsRepository->getSystemSettings();
        $systemSettingsValues = $systemSettings->getPrivateValues();

        try {
            $resolvedValues = Resolver::get($systemSettingsValues, $path);
        } catch (\Exception $e) {
            throw new NotFoundHttpException('Not Found');
        }

        return JsonResponse::create($resolvedValues);
    }
}
