<?php

namespace App\Controller;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use ApiPlatform\Core\Exception\InvalidArgumentException;
use ApiPlatform\Core\Exception\ItemNotFoundException;
use App\Service\BulkRequestService;
use App\Service\MetadataHelper;
use App\EventSubscriber\MqMessagebroadcastSubscriber;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\DependencyInjection\ContainerInterface;

class TypeAgnosticResourceController extends AbstractController
{
    /** @var BulkRequestService $bulkRequestService */
    private $bulkRequestService;

    /** @var MetadataHelper */
    private $metadataHelper;

    /** @var ContainerInterface */
    private $containers;

    public function __construct(BulkRequestService $bulkRequestService, MetadataHelper $metadataHelper, ContainerInterface $containers){
        $this->bulkRequestService = $bulkRequestService;
        $this->metadataHelper = $metadataHelper;
        $this->containers = $containers;
    }

    public function createRecords(Request $request){
        $auditData = [];
        $replicationDataPost = [];
        $replicationDataPut = [];
        $replicationDataDelete = [];
        $objectData = json_decode($request->getContent(), true);
        $headers = $request->headers->all();
        $queryParams = $request->query->all();
        $failedObj = [];
        $successObj = [];
        foreach ($objectData as $k => $item) {
            if (!array_key_exists('@type', $item)) {
                throw new InvalidArgumentException('The "@type" must be specified for all models.');
            }
            $this->bulkRequestService->updateReferenceToIriAndCheckForRelation($item);        
            if (array_key_exists( '@id', $item)){
                $method = 'PUT';
                $subrequestPath = $item['@id'];
            } else {
                try{
                    $routeName = $this->metadataHelper->getTypeFromShortName($item['@type']);
                    $subrequestPath = '/api/3/'. $routeName;       
                    $method = 'POST';
                } catch(ItemNotFoundException $infe){}
            }
            // workflows have an issue with cascade persist unless the container is reinitialized
            $reinitializeKernel = false;
            if ($item['@type'] === 'Workflow' || $item['@type'] === 'WorkflowCollection'){
                $reinitializeKernel = true;
            }
            unset($item['@type']);
            $response = $this->bulkRequestService->executeSubRequest(
                $k,
                $subrequestPath,
                $method,
                $headers,
                $queryParams,
                json_encode($item),
                $auditData,
                $replicationDataPost,
                $replicationDataPut,
                $replicationDataDelete,
                'agnostic_bulk',
                $reinitializeKernel
            );
            if ($response->getStatusCode() == Response::HTTP_CREATED || $response->getStatusCode() == Response::HTTP_OK){
                $successObj[] = json_decode($response->getContent(), true);
            } else {
                $failedObj[] = "Insert for object at index #" .$k. " in the request payload failed with error: " . $response->getContent();
                // entity manager gets closed if an exception occurs. So proceeding to the next set of objects will not work
                break;
            }
        }
        if (count($failedObj) == 0) {
            $responseCode = Response::HTTP_OK;
        } else {
            $responseCode = Response::HTTP_BAD_REQUEST;
        }
        $response = ['persisted' => $successObj, 'failed' => $failedObj];
        $this->containers->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->setContentType('application/json');
        $this->containers->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->publish(json_encode($auditData), MqMessagebroadcastSubscriber::DEFALUT_KEY);
        // type agnostic sends only audit data. doesn't send data to tenant node
        return JsonResponse::create($response, $responseCode);
    }
}