<?php

namespace App\EventSubscriber;

use App\Entity\Core\PgFile;
use App\Entity\Core\Widget;
use App\Service\UtilityService;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\DependencyInjection\ContainerInterface;

class WidgetListener
{
    const WIDGET_PATH = '/opt/cyops-ui/widgets';

    protected $container;

    protected $utilityService;

    public function __construct(ContainerInterface $containers, UtilityService $utilityService)
    {
        $this->container = $containers;

        $this->utilityService = $utilityService;
    }
    public function preUpdate(Widget $widget)
    {

        if (!$widget->getDraft() && $widget->getInstalled()) {
            $this->utilityService->buildWidgetFiles($widget, self::WIDGET_PATH, $this->container->get('doctrine')->getManagerForClass(PgFile::class),'update');
            $this->broadcastBuild($widget->getId());
            $this->utilityService->removeWidgetPreviewFiles($widget->getName());
        } elseif ($widget->getDraft() && !$widget->getInstalled()) {
            $this->utilityService->removeWidgetFiles($widget);
            $this->broadcastBuild(null, $widget->getId());
        }
    }

    public function postRemove(Widget $widget)
    {
        if (!$widget->getDraft()) {
            $this->utilityService->removeWidgetFiles($widget);
            $this->broadcastBuild(null, $widget->getId());
        }
    }
    public function prePersist(Widget $widget)
    {
        if (!$widget->getDraft()) {
            $this->utilityService->buildWidgetFiles($widget, self::WIDGET_PATH, $this->container->get('doctrine')->getManagerForClass(PgFile::class),'create');
            $this->broadcastBuild($widget->getId());
        }
    }

    private function broadcastBuild($wid = null, $deleteWidget = null)
    {
        $dasUri = $this->container->getParameter('das.uri');
        $dasClient = $this->container->get("app.proxy.client.das");
        $cmd = "";
        if ($wid) {
            $cmd = $cmd . "sudo -u nginx php /opt/cyops-api/bin/console cybersponse:widgets:build -w " . $wid;
        } elseif ($deleteWidget) {
            $cmd = $cmd . "sudo -u nginx php /opt/cyops-api/bin/console cybersponse:widgets:build -d " . $wid;
        }

        $objectData = array(
            "commandId" => "cluster_publish_build_widget",
            "command" => $cmd
        );
        $headers = array(
            "Content-type" => "application/json"
        );
        $dasRequests = new GuzzleRequest('post', $dasUri . '/clustercommand/', $headers, json_encode($objectData));
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $resCode = $dasResponse->getStatusCode();
            if ($resCode == 200) {
                return 0;
            }
        } catch (\Exception $error) {
            error_log($error->getMessage());
        }
        return true;
    }
}
