<?php

namespace App\Repository;


use App\Entity\Workflow\Workflow;
use Doctrine\Persistence\ManagerRegistry;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;

/**
 * @method Workflow|null find($id, $lockMode = null, $lockVersion = null)
 * @method Workflow|null findOneBy(array $criteria, array $orderBy = null)
 * @method Workflow[]    findAll()
 * @method Workflow[]    findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
 */
class WorkflowRepository extends ServiceEntityRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, Workflow::class);
    }


    public function getWorkflowsForTriggerType($triggerType, $owners)
    {
        $workflowQueryBuilder = $this->createQueryBuilder('wf');

        if(!$owners){
            /** @var Workflow[] $workflows */
            $workflows = $workflowQueryBuilder
            ->join('wf.triggerStep', 'wfs')
            ->join('wfs.stepType', 'wfst')
            ->where($workflowQueryBuilder->expr()->eq('wfst.name', ':triggerType'))
            ->andWhere($workflowQueryBuilder->expr()->eq('wf.isActive', ':isActive'))
            ->setParameter('triggerType', $triggerType)
            ->setParameter('isActive', true)
            ->getQuery()
            ->execute();
        }else{
            /** @var Workflow[] $workflows */
            $workflows = $workflowQueryBuilder
            ->join('wf.triggerStep', 'wfs')
            ->join('wfs.stepType', 'wfst')
            ->leftJoin('wf.owners', 'wfo')
            ->where($workflowQueryBuilder->expr()->eq('wfst.name', ':triggerType'))
            ->andWhere($workflowQueryBuilder->expr()->eq('wf.isActive', ':isActive'))
            ->andwhere($workflowQueryBuilder->expr()->orX(
                $workflowQueryBuilder->expr()->eq('wf.isPrivate', ':isPrivate'),
                $workflowQueryBuilder->expr()->in('wfo', ':owners'))
            )
            ->setParameter('triggerType', $triggerType)
            ->setParameter('isActive', true)
            ->setParameter('isPrivate', false)
            ->setParameter('owners', $owners)
            ->getQuery()
            ->execute();
        }

        return $workflows;
    }
}
