<?php

namespace App\Security\Voter;

use App\Providers\PermissionResolver;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Authorization\Voter\Voter;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;

class FsrVoter extends Voter
{

    protected $permissionResolver ;

    public function __construct(PermissionResolver $permissionResolver)
    {
        $this->permissionResolver = $permissionResolver;
    }
    
    protected function supports($attribute, $subject)
    {
        return true;
    }

    public function isGranted($attribute) {
        return $this->permissionResolver->resolvePermission($attribute);
    }
    
    protected function voteOnAttribute($attribute, $subject, TokenInterface $token)
    {
        return $this->isGranted($attribute);
    }

    protected function extractParams($attribute){
        return explode('.',$attribute);
    }

    public function denyUnlessAccesssGranted($attribute){
        if (!($this->isGranted($attribute))){
            throw new AccessDeniedException();
        }
    }

}
