<?php

namespace App\Serializer\Normalizer;

use App\Entity\Settings\UserSettings;

final class ActorNormalizer extends EfficientItemNormalizer
{

    public function normalize($object, $format = null, array $context = [])
    {
        $context['ignore_depth'] = false;
        $context['ignore_same_child_resource'] = true;
        // for legacy support, when actor is normalized directly (not as an attribute of another record), the related records are sent as IRIs
        if (!(array_key_exists('normalize_object_class', $context))){
            $context['asJson'] = true;
        }
        $data = parent::normalize($object, $format, $context);
        if (is_array($data) && !$object instanceof \App\Entity\Custom\Appliance) {
            $userSettingsResource = UserSettings::class;
            $userSettingsIri = $this->iriConverter->getIriFromResourceClass($userSettingsResource);
            $data['@settings'] = sprintf('%s/%s', $userSettingsIri, $data['uuid']);
        }
        return $data;
    }

    public function supportsNormalization($data, $format = null): bool
    {
        return 'jsonld' === $format && 
            ($data instanceof \App\Entity\Actors\Actor || $data instanceof \App\Entity\Custom\Person || $data instanceof \App\Entity\Custom\Appliance);
    }

    /**
     * {@inheritdoc}
     */
    public function supportsDenormalization($data, $type, $format = null): bool
    {
        return false;
    }

    public function denormalize($data, $class, $format = null, array $context = [])
    {
        return $this->normalizer->denormalize($data, $class, $format, $context);
    }

}
