<?php
/* Copyright start
Copyright (C) 2008 - 2021 Fortinet Inc.
All rights reserved.
FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
Copyright end */

namespace App\Service;

use App\EventSubscriber\MqMessagebroadcastSubscriber;
use App\Kernel;
use App\Utility\MsspUtility;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use Psr\Container\ContainerInterface;
use Symfony\Component\HttpKernel\HttpKernel;

class BulkRequestService
{
    /**
     * @var HttpKernelInterface
     */
    protected $kernel;

    /** @var ContainerInterface */
    private $container;

    /** @var MSSPUtility */
    private $msspUtility;

    public function __construct(HttpKernelInterface $kernel, ContainerInterface $container, MsspUtility $msspUtility)
    {
        $this->kernel = $kernel;
        $this->container = $container;
        $this->msspUtility = $msspUtility;
    }

    public function prepareRecordSubRequest($resourceClass, $objectData, $subrequestPath, $headers, $queryParams, $method = 'POST', $reinitializeKernel = false)
    {
        $successObj = [];
        $failedObj = [];
        $auditData = [];
        $replicationDataPost = [];
        $replicationDataPut = [];
        $replicationDataDelete = [];
        if (array_key_exists("data", $objectData)) {
            if (isset($objectData->data[0]->{'@type'}) && ($objectData->data[0]->{'@type'} === 'Workflow' || $objectData->data[0]->{'@type'} === 'WorkflowCollection')){
                $reinitializeKernel = true;
            }
            foreach ($objectData->data as $k => $item) {
                // TODO: check what this function does
                $this->updateReferenceToIriAndCheckForRelation($item);
                $response = $this->executeSubRequest(
                    $k,
                    $subrequestPath,
                    $method,
                    $headers,
                    $queryParams,
                    json_encode($item),
                    $auditData,
                    $replicationDataPost,
                    $replicationDataPut,
                    $replicationDataDelete,
                    'bulk',
                    $reinitializeKernel
                );
                if ($response->getStatusCode() == Response::HTTP_CREATED || $response->getStatusCode() == Response::HTTP_OK || $response->getStatusCode() == Response::HTTP_NO_CONTENT) {
                    $successObj[] = json_decode($response->getContent(), true);
                } else {
                    $failedObj[] = $method." method for object at index #" . $k . " in the request payload failed with error: " . $response->getContent();
                }
            }
            // Mq Publish Data
        } else {
            throw new HttpException(Response::HTTP_BAD_REQUEST, 'Invalid query parameters.');
        }
        return $this->processResults($resourceClass, $objectData, $successObj, $failedObj, $auditData, $replicationDataPost, $replicationDataPut, $replicationDataDelete);
    }

    public function prepareUpdateDeleteSubRequest($resourceClass, $objectData, $subrequestPath, $headers, $queryParams, $method = 'DELETE', $reinitializeKernel = false)
    {
        $successObj = [];
        $failedObj = [];
        $auditData = [];
        $replicationDataPost = [];
        $replicationDataPut = [];
        $replicationDataDelete = [];
        if (array_key_exists("data", $objectData)) {
            if (isset($objectData->data[0]->{'@type'}) && ($objectData->data[0]->{'@type'} === 'Workflow' || $objectData->data[0]->{'@type'} === 'WorkflowCollection')){
                $reinitializeKernel = true;
            }
            foreach ($objectData->data as $k => $item) {
                // PUT needs @id. if UUID only, generate @id from it
                if (!isset($item->{'@id'}) && isset($item->{'uuid'})){
                    $item->{'@id'} = $subrequestPath . '/' .$item->{'uuid'};
                }
                $path = $item->{'@id'};
                $this->updateReferenceToIriAndCheckForRelation($item);
                $response = $this->executeSubRequest(
                    $k,
                    $path,
                    $method,
                    $headers,
                    $queryParams,
                    json_encode($item),
                    $auditData,
                    $replicationDataPost,
                    $replicationDataPut,
                    $replicationDataDelete,
                    'bulk',
                    $reinitializeKernel
                );
                if ($response->getStatusCode() == Response::HTTP_CREATED || $response->getStatusCode() == Response::HTTP_OK || $response->getStatusCode() == Response::HTTP_NO_CONTENT) {
                    $successObj[] = json_decode($response->getContent(), true);
                } else {
                    $failedObj[] = $method." method for object at index #" . $k . " in the request payload failed with error: " . $response->getContent();
                }
            }
        } else {
            throw new HttpException(Response::HTTP_BAD_REQUEST, 'Invalid query parameters.');
        }

        if ($method == Request::METHOD_DELETE) {
            $responseCode = Response::HTTP_NO_CONTENT;
        } else {
            $responseCode = Response::HTTP_OK;
        }

        return $this->processResults($resourceClass, $objectData, $successObj, $failedObj, $auditData, $replicationDataPost, $replicationDataPut, $replicationDataDelete, $responseCode);
    }

    private function processResults($resourceClass, $objectData, $successObj, $failedObj, $auditData, $replicationDataPost, $replicationDataPut, $replicationDataDelete, $responseCode = Response::HTTP_CREATED)
    {
        $this->container->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->setContentType('application/json');
        $this->container->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->publish(json_encode($auditData), MqMessagebroadcastSubscriber::DEFALUT_KEY);

        if (!empty($replicationDataPost)){
            $this->msspUtility->replicateData($resourceClass, $objectData, $replicationDataPost, 'POST');
        }
        if (!empty($replicationDataPut)){
            $this->msspUtility->replicateData($resourceClass, $objectData, $replicationDataPut, 'PUT');
        }
        if (!empty($replicationDataDelete)){
            $this->msspUtility->replicateData($resourceClass, $objectData, $replicationDataDelete, 'DELETE');
        }
        $data = ['@type' => 'hydra:Collection', 'hydra:member' => $successObj, 'hydra:totalItems' => sizeof($successObj)];
        if (count($failedObj) == 0) {
            return JsonResponse::create($data, $responseCode);
        } else {
            $partialUpdatedObject = array();
            $partialUpdatedObject["success"] = $successObj;
            $partialUpdatedObject["failure"] = $failedObj;
            $responseCode = Response::HTTP_MULTI_STATUS;
            if (count($successObj) == 0) {
                $responseCode = Response::HTTP_INTERNAL_SERVER_ERROR;
            }
            // TODO: check if partial response is also returned as hydra collection
            return JsonResponse::create($partialUpdatedObject, $responseCode);
        }
    }

    public function executeSubRequest(int $index, string $path, string $method, array $headers, array $queryParams, string $body, 
        & $audit_data, & $replicationDataPost, & $replicationDataPut, & $replicationDataDelete, string $subrequestType='bulk', $reinitializeKernel = false): Response
    {
        $subRequest = Request::create($path, $method, $queryParams, [], [], [], $body);
        $subRequest->headers->replace($headers);
        $subRequest->query->replace($queryParams);
        $subRequest->attributes->set('fsr_subrequest_type', $subrequestType);
        try {
            if ($reinitializeKernel === true){
                $kernelSubRequest = new Kernel('prod', false);
                $response = $kernelSubRequest->handle($subRequest, HttpKernelInterface::MASTER_REQUEST, true);
            } else {
                $httpKernel = $this->container->get('http_kernel');
                $response = $httpKernel->handle($subRequest, HttpKernelInterface::MASTER_REQUEST, true);
            }
            
            if ($subRequest->attributes->has('fsr_audit_data')){
                array_push($audit_data, $subRequest->attributes->get('fsr_audit_data'));
            }
            if ($subRequest->attributes->has('fsr_replication_data')){
                $replicationData = $subRequest->attributes->get('fsr_replication_data');
                if ($replicationData->{'method'} == 'POST'){
                    array_push($replicationDataPost, $replicationData);
                } else if ($replicationData->{'method'} == 'PUT'){
                    array_push($replicationDataPut, $replicationData);
                } else if ($replicationData->{'method'} == 'DELETE'){
                    array_push($replicationDataDelete, $replicationData);
                }
            }            
            if ($reinitializeKernel === true){
                $kernelSubRequest->terminate($subRequest, $response);
                $kernelSubRequest->shutdown();
            } else {
                $httpKernel->terminate($subRequest, $response);
            }
            return $response;
        } catch (\Exception $e) {
            return Response::create(sprintf('Batch element #%d failed, check the log files.', $index), 400);
        }
    }

    /**
     * @param $request_body
     * @return mixed
     */
    public function updateReferenceToIriAndCheckForRelation(&$request_body)
    {
        foreach ($request_body as $key => $value) {
            if (gettype($value) == "object" && isset($request_body->$key->{'@id'})) {
                $request_body->$key = $request_body->$key->{'@id'};
            }
        }
    }

    public function buildNormalizationContextFromQuery(array $queryParams): array
    {
        $context = [];
        foreach ($queryParams as $queryParam => $value) {
            if (str_starts_with($queryParam, "$")) {
                $key = substr($queryParam, 1);
                $context[$key] = $value;
            }
        }
        return $context;
    }
}
