<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service\ConfigExportImport;

use App\Entity\ViewTemplate\SystemViewTemplate;

/**
 * Class ViewConfig
 * @package App\Service\ConfigExportImport
 */
class ViewConfig extends PortingConfig
{
    /**
     * @param array $allData
     * @param string $key
     * @return void
     */
    public function getImportOptions(array $allData, string $key, $tokenType)
    {
        $data = $allData[$key];
        $options = [];
        $repository = $this->loadRepository(SystemViewTemplate::class);
        foreach ($data as $key => $value) {
            if (!$value) {
                continue;
            }
            if (count(array_filter(array_keys($value), 'is_string')) == 0) {
                // This is a sequential array, used by legacy module view templates
                foreach ($value as $view) {
                    $existing = $repository->findOneBy(['uuid' => $view['uuid']]);
                    $individualOptions = [];
                    $uuid = $view['uuid'];
                    if (!$uuid) {
                        $uuid = implode('-', explode('/', $view['id']));
                    }
                    $individualOptions['uuid'] = $uuid;
                    if ($view['config'] && array_key_exists('title', $view['config'])) {
                        $individualOptions['name'] = $view['config']['title'];
                    }
                    $individualOptions['module'] = $key;
                    $individualOptions['exists'] = $existing ? true : false;
                    $individualOptions['include'] = true;

                    $options[] = $individualOptions;
                }
                continue;
            }
            $existing = $repository->findOneBy(['uuid' => $value['uuid']]);
            $individualOptions = [];
            $uuid = $value['uuid'];
            $pieces = explode('-', $value['uuid']);
            if (count($pieces) === 3) {
                $individualOptions['module'] = $pieces[1];
            }
            if ($value['config'] && array_key_exists('title', $value['config'])) {
                $individualOptions['name'] = $value['config']['title'];
            }
            $individualOptions['uuid'] = $uuid;
            $individualOptions['exists'] = $existing ? true : false;
            $individualOptions['include'] = true;

            $options[] = $individualOptions;
        }
        return $options;
    }

    /**
     * Imports the config data using the specified options
     *
     * @param array $data
     * @param array $options
     * @return mixed
     */
    public function import(array $data, array $options, $tokenType) {
        $toAdd = [];
        $toUpdate = [];
        foreach ($options as $option) {
            if (!$option['include']) {
                // Do not merge skipped items
                continue;
            }
            $foundView = null;
            foreach ($data as $value) {
                if (!$value) {
                    continue;
                }
                if (count(array_filter(array_keys($value), 'is_string')) == 0) {
                    // This is a sequential array, used by module view templates
                    foreach ($value as $view) {
                        if ($option['uuid'] == $view['uuid']) {
                            $foundView = $view;
                            break;
                        }
                    }
                    continue;
                }
                if ($option['uuid'] == $value['uuid']) {
                    $foundView = $value;
                    break;
                }
            }
            if ($option['exists'] && $foundView) {
                $toUpdate[] = $foundView;
            } else if ($foundView) {
                $toAdd[] = $foundView;
            }
        }
        $results = [];
        if (count($toAdd) > 0) {
            $fullURI = sprintf('%s/api/3/', $this->container->getParameter('crudhub.uri'));
            $result = $this->sendRequest($fullURI, 'post', $toAdd, $tokenType);
            $results[] = $result;
        }
        if (count($toUpdate) > 0) {
            $fullURI = sprintf('%s/api/3/update/system_view_templates', $this->container->getParameter('crudhub.uri'));
            $body = [
              "data" => $toUpdate
            ];
            $result = $this->sendRequest($fullURI, 'put', $body, $tokenType);
            $results[] = $result;
        }
        
        if (in_array(false, $results)) {
            return false;
        }
        return true;
    }

    /**
     * Exports the config using the specified options
     *
     * @param array $options
     * @return array
     */
    public function export(array $options, $tokenType='JWT') {
        return $this->getResourceFromIRI($options, SystemViewTemplate::class);
    }
}
