<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service\ConfigExportImport;

use App\Entity\Core\Widget;
use App\Security\Voter\FsrVoter;
use Doctrine\ORM\EntityManagerInterface;
use App\Service\ConfigExportImport\PortingConfig;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

/**
 * Class WidgetConfig
 */
class WidgetConfig extends PortingConfig
{
    private $fsrVoter;

    public function __construct(EntityManagerInterface $entityManager, NormalizerInterface $normalizer, 
        TokenStorageInterface $tokenStorage, ContainerInterface $container, FsrVoter $fsrVoter)
    {
        $this->fsrVoter = $fsrVoter;
        parent::__construct($entityManager, $normalizer, $tokenStorage, $container);
    }

    /**
     *
     * @param array $data Playbook object containing collections & global vars
     * @return array
     */
    public function getImportOptions(array $allData, string $key, $tokenType)
    {
        $data = $allData[$key];
        if ($tokenType && ! ($this->fsrVoter->isGranted('update.widgets') || $this->fsrVoter->isGranted('create.widgets') || $this->fsrVoter->isGranted('read.widgets'))) {
            return [];
        }
        $options = [];
        $repository = $this->loadRepository(Widget::class);
        $existingWidgets = $repository->findAll();
        $existingWidgetNames = array_map(function($item) {
            return $item->getName();
        }, $existingWidgets);
        $options = [];
        foreach ($data as $widget) {
            $existing = in_array($widget['name'], $existingWidgetNames);
            $individualOptions = [];
            $individualOptions['name'] = $widget['name'];
            $individualOptions['version'] = $widget['version'];
            $individualOptions['title'] = $widget['title'];
            $individualOptions['exists'] = $existing;
            $individualOptions['include'] = !$existing;

            $options[] = $individualOptions;
        }
        return $options;
    }
    
    
    /**
     * Imports the config data using the specified options
     *
     * @param array $data
     * @param array $options
     * @return mixed
     */
    public function import(array $data, array $options, $tokenType)
    {
        foreach ($options as $option) {
            if ($option['exists'] && $option['include']) {
                // Skip existing widgets
                continue;
            }
            // Install & wait if not exists
            $params = [
                'name' => $option['name'],
                'version' => $option['version'],
                'replace' => true
            ];
            $this->sendRequest(sprintf('%s/api/3/install/widget', $this->container->getParameter('crudhub.uri')), 'post', $params, $tokenType);
        }
        return true;
    }

    /**
     * Exports the config using the specified options
     *
     * @param array $options
     * @return array
     */
    public function export(array $options, $tokenType='JWT')
    {
        if (!$this->fsrVoter->isGranted('read.widgets')) {
            return [];
        }
        $data = [];
        if (count($options) > 0) {
            $repository = $this->loadRepository(Widget::class);
            $widgets = $repository->findAll();
        }
        foreach ($options as $option) {
            foreach ($widgets as $widget) {
                // Load ingestion playbooks
                if ($option == $widget->getName()) {
                    $widgetObject = [
                        'name' => $widget->getName(),
                        'version' => $widget->getVersion(),
                        'title' => $widget->getTitle()
                    ];
                    
                    $data[] = $widgetObject;
                    break;
                }
            }
        }
        return $data;
    }
}
