<?php

namespace App\Service;


use Ramsey\Uuid\Uuid;
use App\Entity\Actors\Actor;
use Doctrine\ORM\EntityManager;
use App\Entity\Authorization\Role;
use App\Entity\Authorization\Team;
use ApiPlatform\Core\Api\IriConverterInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class SamlActorService
{

    protected $container;

    /** @var EntityManager */
    protected $entityManager;

    /** @var TokenStorageInterface */
    protected $tokenStorage;

    /** @var IriConverterInterface */
    protected $iriConverter;

    /** @var ProxyRepository */
    protected $proxyRepository;

    public function __construct(TokenStorageInterface $tokenStorage, IriConverterInterface $iriConverter, ContainerInterface $container)
    {
        $this->tokenStorage = $tokenStorage;
        $this->iriConverter = $iriConverter;
        $this->container = $container;
        $this->entityManager = $this->container->get('doctrine.orm.entity_manager');
        $this->proxyRepository = $this->entityManager->getRepository(Actor::class);
    }

    /**
     * @return string
     */
    public function getCurrentUserAsIri()
    {
        $user = $this->tokenStorage->getToken()->getUser();
        return $this->iriConverter->getIriFromItem($user);
    }

    /**
     *
     * @param $className string
     * @param mixed $actorBody
     * @return boolean
     *
     */
    public function saveSamlUser($className, $actorBody)
    {
        $timestamp = (new \DateTime('NOW', new \DateTimeZone('UTC')))->getTimestamp();
        $userAttributes = $actorBody;
        $userId = null;
        if (array_key_exists('userId', $userAttributes)) {
            $userId = $userAttributes['userId'];
        }
        $actor = $this->getActor($userId);
        //Check if user exist
        if ($actor === null) {
            $actor = new $className();
            $actor->setUuid(Uuid::uuid4()->toString());
            $actor->setCreateDate($timestamp);
            $actor->setUserId($userId);
            $actor->setUserType('SAML');

            if (array_key_exists('teams', $userAttributes)) {
                $teamIds = $userAttributes['teams'];
                foreach ($teamIds as $teamId) {
                    $team = $this->getTeam($teamId);
                    if ($team !== null) {
                        $actor->addTeam($team);
                    }
                }
            }

            if (array_key_exists('roles', $userAttributes)) {
                $roleIds = $userAttributes['roles'];
                foreach ($roleIds as $roleId) {
                    $role = $this->getRole($roleId);
                    if ($role !== null) {
                        $actor->addRole($role);
                    }
                }
            }
        }
        if (array_key_exists('roles', $userAttributes)) {
            $rolesAttribute = $userAttributes['roles'];
        }
        if (array_key_exists('teams', $userAttributes)) {
            $teamsAttributes = $userAttributes['teams'];
        }
        $overrideRbacFlag = $userAttributes['overrideRbac'];
        $userAttributes = $userAttributes['attributes'];
        if (array_key_exists('firstName', $userAttributes)) {
            $actor->setFirstname($userAttributes['firstName']);
        } else {
            $actor->setFirstname($userId);
        }
        if (array_key_exists('lastName', $userAttributes)) {
            $actor->setLastname($userAttributes['lastName']);
        }
        if (array_key_exists('email', $userAttributes)) {
            $actor->setEmail($userAttributes['email']);
        }
        if (array_key_exists('title', $userAttributes)) {
            $actor->setTitle($userAttributes['title']);
        }
        if (array_key_exists('department', $userAttributes)) {
            $actor->setDepartment($userAttributes['department']);
        }
        if (array_key_exists('description', $userAttributes)) {
            $actor->setDescription($userAttributes['description']);
        }
        if (array_key_exists('phoneFax', $userAttributes)) {
            $actor->setPhoneFax($userAttributes['phoneFax']);
        }
        if (array_key_exists('phoneHome', $userAttributes)) {
            $actor->setPhoneHome($userAttributes['phoneHome']);
        }
        if (array_key_exists('phoneMobile', $userAttributes)) {
            $actor->setPhoneMobile($userAttributes['phoneMobile']);
        }
        if (array_key_exists('phoneWork', $userAttributes)) {
            $actor->setPhoneWork($userAttributes['phoneWork']);
        }

        if ($overrideRbacFlag) {

            $existingTeams = $actor->getTeams();
            foreach ($existingTeams as $value) {
                $actor->removeTeam($value);
            }

            $existingRoles = $actor->getRoles(true);
            foreach ($existingRoles as $value) {
                $actor->removeRole($value);
            }

            foreach ($teamsAttributes as $teamId) {
                $team = $this->getTeam($teamId);
                if ($team !== null) {
                    $actor->addTeam($team);
                }
            }

            foreach ($rolesAttribute as $roleId) {
                $role = $this->getRole($roleId);
                if ($role !== null) {
                    $actor->addRole($role);
                }
            }
        }

        $actor->setModifyDate($timestamp);
        $this->entityManager->persist($actor);
        $this->entityManager->flush();
        return true;
    }

    /**
     *
     * @param String $userId
     * @return Actor $actor
     */
    public function getActor($userId)
    {
        $criteria = array('userId' => $userId);
        return $this->proxyRepository->findOneBy($criteria);
    }
    /**
     *
     * @param String $teamId
     * @return Team $team
     */
    private function getTeam($teamId)
    {
        $teamRepository = $this->entityManager->getRepository(Team::class);
        return $teamRepository->find($teamId);
    }

    /**
     *
     * @param String $roleId
     * @return Role $role
     */
    private function getRole($roleId)
    {
        $roleRepository = $this->entityManager->getRepository(Role::class);
        return $roleRepository->find($roleId);
    }
}
