<?php

namespace App\Traits;

use App\Traits\Trackable;
use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class TrackableListener
{
    /** @var TokenStorageInterface */
    protected $tokenStorage;

    public function __construct(TokenStorageInterface $tokenStorage)
    {
        $this->tokenStorage = $tokenStorage;
    }

    /**
     * @ORM\PreUpdate
     */
    public function preUpdate(LifecycleEventArgs $args)
    {
        $object = $args->getObject();
        if ($this->isTrackable($object)) {
            list($user, $now) = $this->getTrackableInfo();
            /** @var Trackable $object */
            $object->setModifyDate($now);
            if (!is_null($user)) {
                $object->setModifyUser($user);
            }
        }
    }

    /**
     * @ORM\PrePersist
     */
    public function prePersist(LifecycleEventArgs $args)
    {
        $object = $args->getObject();
        if ($this->isTrackable($object)) {
            list($user, $now) = $this->getTrackableInfo();
            /** @var Trackable $object */
            $object->setCreateDate($now);
            $object->setModifyDate($now);
            if (!is_null($user)) {
                $object->setCreateUser($user);
                $object->setModifyUser($user);
            }
        }
    }

    /**
     * @param $object
     * @return bool
     */
    protected function isTrackable($object)
    {
        return in_array(Trackable::class, class_uses(get_class($object)));
    }

    /**
     * @return array
     */
    private function getTrackableInfo()
    {
        $userToken = $this->tokenStorage->getToken();
        $user = null;
        if (!is_null($userToken)) {
            $user = $userToken->getUser();
        }
        $now = new \DateTime('NOW', new \DateTimeZone('UTC'));
        return array($user, $now);
    }
}
